@@ -6,30 +6,54 @@ import shutil
 import subprocess
 import sys
 import tempfile
+from collections import defaultdict
 from importlib import import_module
 from pathlib import Path
 from textwrap import dedent
-from typing import Dict, Iterator, List, Mapping, Match, Optional, Set, Tuple
+from typing import DefaultDict, Dict, Iterator, List, Optional, Set, Tuple
 
 from pkg_resources import parse_version
 
 from nbqa import config_parser, replace_source, save_source
-from nbqa.cmdline import CLIArgs
-from nbqa.config import Configs
+from nbqa.cmdline import BOLD, RESET, CLIArgs
+from nbqa.config.config import Configs
 from nbqa.find_root import find_project_root
 from nbqa.notebook_info import NotebookInfo
 from nbqa.optional import metadata
+from nbqa.output_parser import map_python_line_to_nb_lines
 
-CONFIG_FILES = ["setup.cfg", "tox.ini", "pyproject.toml"]
+CONFIG_FILES: DefaultDict[str, List[str]] = defaultdict(
+    lambda: ["setup.cfg", "tox.ini", "pyproject.toml"]
+)
+CONFIG_FILES["black"] = ["pyproject.toml"]
+CONFIG_FILES["flake8"] = ["setup.cfg", "tox.ini", ".flake8"]
+CONFIG_FILES["pyupgrade"] = []
+CONFIG_FILES["mypy"] = ["mypy.ini", ".mypy.ini", "setup.cfg"]
+CONFIG_FILES["doctest"] = []
+CONFIG_FILES["isort"] = [
+    ".isort.cfg",
+    "pyproject.toml",
+    "setup.cfg",
+    "tox.ini",
+    ".editorconfig",
+]
+CONFIG_FILES["pylint"] = ["pylintrc", ".pylintrc", "pyproject.toml", "setup.cfg"]
 BASE_ERROR_MESSAGE = dedent(
-    """
-
-    😭 {} 😭
-
-    Please report a bug at https://github.com/nbQA-dev/nbQA/issues 🙏
+    f"""\
+    {BOLD}{{}}
+    Please report a bug at https://github.com/nbQA-dev/nbQA/issues {RESET}
     """
 )
 MIN_VERSIONS = {"isort": "5.3.0"}
+VIRTUAL_ENVIRONMENTS_URL = (
+    "https://realpython.com/python-virtual-environments-a-primer/"
+)
+EXCLUDES = (
+    r"/("
+    r"\.direnv|\.eggs|\.git|\.hg|\.ipynb_checkpoints|\.mypy_cache|\.nox|\.svn|\.tox|\.venv|"
+    r"_build|buck-out|build|dist|venv"
+    r")/"
+)
 
 
 class UnsupportedPackageVersionError(Exception):
@@ -37,11 +61,9 @@ class UnsupportedPackageVersionError(Exception):
 
     def __init__(self, command: str, current_version: str, min_version: str) -> None:
         """Initialise with command, current version, and minimum version."""
-        self.msg = dedent(
-            f"""\
-            nbqa only works with {command} >= {min_version}, while
-            you have {current_version} installed.
-            """
+        self.msg = (
+            f"{BOLD}nbqa only works with {command} >= {min_version}, "
+            f"while you have {current_version} installed.{RESET}"
         )
         super().__init__(self.msg)
 
@@ -63,13 +85,48 @@ def _get_notebooks(root_dir: str) -> Iterator[Path]:
     if not Path(root_dir).is_dir():
         return iter((Path(root_dir),))
     return (
-        i for i in Path(root_dir).rglob("*.ipynb") if ".ipynb_checkpoints" not in str(i)
+        i
+        for i in Path(root_dir).rglob("*.ipynb")
+        if not re.search(EXCLUDES, str(i.resolve().as_posix()))
     )
 
 
-def _get_all_notebooks(root_dirs: List[str]) -> Iterator[Path]:
+def _filter_by_include_exclude(
+    notebooks: Iterator[Path],
+    include: Optional[str],
+    exclude: Optional[str],
+) -> Iterator[Path]:
     """
-    Get generator with all notebooks passed in via the command-line.
+    Include files which match include, exclude those matching exclude.
+
+    notebooks
+        Notebooks (not directories) to run code quality tool on.
+    include:
+        Global file include pattern.
+    exclude:
+        Global file exclude pattern.
+
+    Returns
+    -------
+    Iterator
+        Notebooks matching include and not matching exclude.
+    """
+    include = include or ""
+    exclude = exclude or "^$"
+    include_re, exclude_re = re.compile(include), re.compile(exclude)
+    return (
+        notebook
+        for notebook in notebooks
+        if include_re.search(str(notebook.as_posix()))
+        if not exclude_re.search(str(notebook.as_posix()))
+    )
+
+
+def _get_all_notebooks(
+    root_dirs: List[str], files: Optional[str], exclude: Optional[str]
+) -> Iterator[Path]:
+    """
+    Get generator with all notebooks passed in via the command-line, applying exclusions.
 
     Parameters
     ----------
@@ -81,7 +138,9 @@ def _get_all_notebooks(root_dirs: List[str]) -> Iterator[Path]:
     Iterator
         All Jupyter Notebooks found in all passed directories/notebooks.
     """
-    return (j for i in root_dirs for j in _get_notebooks(i))
+    return _filter_by_include_exclude(
+        (j for i in root_dirs for j in _get_notebooks(i)), files, exclude
+    )
 
 
 def _temp_python_file_for_notebook(
@@ -111,7 +170,9 @@ def _temp_python_file_for_notebook(
         If notebook doesn't exist.
     """
     if not notebook.exists():
-        raise FileNotFoundError(f"No such file or directory: {str(notebook)}")
+        raise FileNotFoundError(
+            f"{BOLD}No such file or directory: {str(notebook)}{RESET}"
+        )
     relative_notebook_path = (
         notebook.resolve().relative_to(project_root).with_suffix(".py")
     )
@@ -120,44 +181,6 @@ def _temp_python_file_for_notebook(
     return temp_python_file
 
 
-def _map_python_line_to_nb_lines(
-    out: str, notebook: Path, cell_mapping: Mapping[int, str]
-) -> str:
-    """
-    Make sure stdout and stderr make reference to Jupyter Notebook cells and lines.
-
-    Parameters
-    ----------
-    out
-        Captured stdout from third-party tool.
-    notebook
-        Original Jupyter notebook.
-    cell_mapping
-        Mapping from Python file lines to Jupyter notebook cells.
-
-    Returns
-    -------
-    str
-        Stdout with references to temporary Python file's lines replaced with references
-        to notebook's cells and lines.
-    """
-    pattern = rf"(?<={notebook.name}:)\d+"
-
-    def substitution(match: Match[str]) -> str:
-        """Replace Python line with corresponding Jupyter notebook cell."""
-        return str(cell_mapping[int(match.group())])
-
-    out = re.sub(pattern, substitution, out)
-
-    # doctest pattern
-    pattern = rf'(?<={notebook.name}", line )\d+'
-    if re.search(pattern, out) is not None:
-        out = re.sub(pattern, substitution, out)
-        out = out.replace(f'{notebook.name}", line ', f'{notebook.name}", ')
-
-    return out
-
-
 def _replace_temp_python_file_references_in_out_err(
     tmpdirname: str,
     temp_python_file: Path,
@@ -240,21 +263,20 @@ def _create_blank_init_files(
 
 
 def _preserve_config_files(
-    nbqa_config: Optional[str], tmpdirname: str, project_root: Path
+    config_files: List[str], tmpdirname: str, project_root: Path
 ) -> None:
     """
     Copy local config file to temporary directory.
 
     Parameters
     ----------
-    nbqa_config
-        Config file for third-party tool (e.g. mypy).
+    config_files
+        Config files for third-party tool (e.g. mypy).
     tmpdirname
         Temporary directory to store converted notebooks in.
     project_root
         Root of repository, where .git / .hg / .nbqa.ini file is.
     """
-    config_files = [nbqa_config] if nbqa_config is not None else CONFIG_FILES
     for config_file in config_files:
         config_file_path = project_root / config_file
         if config_file_path.exists():
@@ -411,14 +433,15 @@ def _run_command(
         stdout=subprocess.PIPE,
         cwd=tmpdirname,
         env=env,
+        universal_newlines=True,  # from Python3.7 this can be replaced with `text`
     )
 
     mutated = [_get_mtimes(i) for i in args] != before
 
     output_code = output.returncode
 
-    out = output.stdout.decode()
-    err = output.stderr.decode()
+    out = output.stdout
+    err = output.stderr
 
     return out, err, output_code, mutated
 
@@ -437,21 +460,21 @@ def _get_command_not_found_msg(command: str) -> str:
         Message to display to stdout.
     """
     template = dedent(
-        """\
-        Command `{command}` not found by nbqa.
+        f"""\
+        {BOLD}Command `{command}` not found by nbqa.{RESET}
 
         Please make sure you have it installed in the same Python environment as nbqa. See
-        e.g. https://realpython.com/python-virtual-environments-a-primer/ for how to set up
+        e.g. {VIRTUAL_ENVIRONMENTS_URL} for how to set up
         a virtual environment in Python.
 
-        Since nbqa is installed at {nbqa_loc} and uses the Python executable found at
-        {python}, you could fix this issue by running `{python} -m pip install {command}`.
+        Since nbqa is installed at {{nbqa_loc}} and uses the Python executable found at
+        {{python}}, you could fix this issue by running `{{python}} -m pip install {command}`.
         """
     )
     python_executable = sys.executable
     nbqa_loc = str(Path(sys.modules["nbqa"].__file__).parent)
 
-    return template.format(command=command, python=python_executable, nbqa_loc=nbqa_loc)
+    return template.format(python=python_executable, nbqa_loc=nbqa_loc)
 
 
 def _get_configs(cli_args: CLIArgs, project_root: Path) -> Configs:
@@ -513,10 +536,24 @@ def _run_on_one_root_dir(
 
         nb_to_py_mapping = {
             notebook: _temp_python_file_for_notebook(notebook, tmpdirname, project_root)
-            for notebook in _get_all_notebooks(cli_args.root_dirs)
+            for notebook in _get_all_notebooks(
+                cli_args.root_dirs, configs.nbqa_files, configs.nbqa_exclude
+            )
         }
 
-        _preserve_config_files(configs.nbqa_config, tmpdirname, project_root)
+        if not nb_to_py_mapping:
+            sys.stderr.write(
+                "No .ipynb notebooks found in given directories: "
+                f"{' '.join(i for i in cli_args.root_dirs if Path(i).is_dir())}\n"
+            )
+            return 0
+
+        config_files = (
+            [configs.nbqa_config]
+            if configs.nbqa_config
+            else CONFIG_FILES[cli_args.command]
+        )
+        _preserve_config_files(config_files, tmpdirname, project_root)
 
         nb_info_mapping: Dict[Path, NotebookInfo] = {}
 
@@ -548,19 +585,32 @@ def _run_on_one_root_dir(
             out, err = _replace_temp_python_file_references_in_out_err(
                 tmpdirname, temp_python_file, notebook, out, err
             )
-            out = _map_python_line_to_nb_lines(
-                out, notebook, nb_info_mapping[notebook].cell_mappings
-            )
+            try:
+                out, err = map_python_line_to_nb_lines(
+                    cli_args.command,
+                    out,
+                    err,
+                    notebook,
+                    nb_info_mapping[notebook].cell_mappings,
+                )
+            except Exception as exc:  # pylint: disable=W0703
+                msg = (
+                    f"{repr(exc)} while parsing output "
+                    f"from applying {cli_args.command} to {str(notebook)}"
+                )
+                sys.stderr.write(BASE_ERROR_MESSAGE.format(msg))
 
             if mutated:
                 if not configs.nbqa_mutate:
+                    # pylint: disable=C0301
                     msg = dedent(
                         f"""\
-                        💥 Mutation detected, will not reformat! Please use the `--nbqa-mutate` flag:
+                        {BOLD}Mutation detected, will not reformat! Please use the `--nbqa-mutate` flag, e.g.:{RESET}
 
-                            {" ".join([str(cli_args), "--nbqa-mutate"])}
+                            nbqa {cli_args.command} notebook.ipynb --nbqa-mutate
                         """
                     )
+                    # pylint: enable=C0301
                     raise SystemExit(msg)
 
                 try:
