@@ -1,13 +1,38 @@
 """Parses the command line arguments provided."""
 import argparse
-import sys
 from textwrap import dedent
 from typing import List, Optional
 
 from nbqa import __version__
 
+BOLD = "\033[1m"
+RESET = "\x1b[0m"
+CONFIGURATION_URL = "https://nbqa.readthedocs.io/en/latest/configuration.html"
+DOCS_URL = "https://nbqa.readthedocs.io/en/latest/index.html"
+USAGE_MSG = dedent(
+    f"""\
+    nbqa <code quality tool> <notebook or directory> <nbqa options> <code quality tool arguments>
 
-class CLIArgs:
+    {BOLD}Please specify:{RESET}
+    - 1) a code quality tool (e.g. `black`, `pyupgrade`, `flake`, ...)
+    - 2) some notebooks (or, if supported by the tool, directories)
+    - 3) (optional) flags for nbqa (e.g. `--nbqa-mutate`)
+    - 4) (optional) flags for code quality tool (e.g. `--line-length` for `black`)
+
+    {BOLD}Examples:{RESET}
+        nbqa flake8 notebook.ipynb
+        nbqa black notebook.ipynb --line-length=96
+        nbqa pyupgrade notebook_1.ipynb notebook_2.ipynb
+
+    {BOLD}Mutation:{RESET} to let `nbqa` modify your notebook(s), also pass `--nbqa-mutate`, e.g.:
+        nbqa black notebook.ipynb --nbqa-mutate
+
+    See {DOCS_URL} for more details on how to run `nbqa`.
+    """
+)
+
+
+class CLIArgs:  # pylint: disable=R0902
     """
     Stores the command line arguments passed.
 
@@ -25,6 +50,10 @@ class CLIArgs:
         Ignore cells whose first line starts with the input token
     nbqa_addopts
         Any additional flags passed to third-party tool (e.g. :code:`--quiet`).
+    nbqa_files
+        Global file include pattern.
+    nbqa_exclude
+        Global file exclude pattern.
     """
 
     command: str
@@ -33,6 +62,8 @@ class CLIArgs:
     nbqa_mutate: bool
     nbqa_ignore_cells: Optional[str]
     nbqa_config: Optional[str]
+    nbqa_files: Optional[str]
+    nbqa_exclude: Optional[str]
 
     def __init__(self, args: argparse.Namespace, cmd_args: List[str]) -> None:
         """
@@ -51,6 +82,8 @@ class CLIArgs:
         self.nbqa_mutate = args.nbqa_mutate or False
         self.nbqa_config = args.nbqa_config or None
         self.nbqa_ignore_cells = args.nbqa_ignore_cells or None
+        self.nbqa_files = args.nbqa_files or None
+        self.nbqa_exclude = args.nbqa_exclude or None
 
     def __str__(self) -> str:
         """Return the command from the parsed command line arguments."""
@@ -62,6 +95,10 @@ class CLIArgs:
             args.append(f"--nbqa-config={self.nbqa_config}")
         if self.nbqa_ignore_cells:
             args.append(f"--nbqa-ignore-cells={self.nbqa_ignore_cells}")
+        if self.nbqa_files:
+            args.append(f"--nbqa-files={self.nbqa_files}")
+        if self.nbqa_exclude:
+            args.append(f"--nbqa-exclude={self.nbqa_exclude}")
         args.extend(self.nbqa_addopts)
 
         return " ".join(args)
@@ -79,26 +116,23 @@ class CLIArgs:
         -------
         CLIArgs
             Object that holds all the parsed command line arguments.
-        Raises
-        ------
-        ValueError
-            If user doesn't specify both a command and a notebook/directory to run it
-            on (e.g. if the user runs :code:`nbqa flake8` instead of :code:`nbqa flake8 .`).
         """
         parser = argparse.ArgumentParser(
-            description="Adapter to run any code-quality tool on a Jupyter notebook.",
-            usage=dedent(
-                """\
-                nbqa <command> <notebook or directory> <flags>
-                example:
-                    nbqa flake8 my_notebook.ipynb --ignore=E203\
-                """
-            ),
+            description="Run any standard Python code-quality tool on a Jupyter notebook.",
+            usage=USAGE_MSG,
         )
         parser.add_argument("command", help="Command to run, e.g. `flake8`.")
         parser.add_argument(
             "root_dirs", nargs="+", help="Notebooks or directories to run command on."
         )
+        parser.add_argument(
+            "--nbqa-files",
+            help="Global file include pattern.",
+        )
+        parser.add_argument(
+            "--nbqa-exclude",
+            help="Global file exclude pattern.",
+        )
         parser.add_argument(
             "--nbqa-mutate",
             action="store_true",
@@ -123,21 +157,5 @@ class CLIArgs:
         parser.add_argument(
             "--version", action="version", version=f"nbqa {__version__}"
         )
-        try:
-            args, cmd_args = parser.parse_known_args(argv)
-        except SystemExit as exception:
-            if exception.code != 0:
-                msg = dedent(
-                    """\
-                    Please specify both a command and a notebook/directory.
-                    e.g nbqa flake8 my_notebook.ipynb
-
-                    To know all the options supported by nbqa, use `nbqa --help`. To
-                    read in detail about the various configuration options supported by
-                    nbqa, refer to https://nbqa.readthedocs.io/en/latest/configuration.html
-                    """
-                )
-                raise ValueError(msg) from exception
-            sys.exit(0)  # pragma: nocover
-        else:
-            return CLIArgs(args, cmd_args)
+        args, cmd_args = parser.parse_known_args(argv)
+        return CLIArgs(args, cmd_args)
